@@ -3,8 +3,8 @@
  */
 
 /*
-Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2006, 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
-Contributed by the Arenaire and Cacao projects, INRIA.
+Copyright 1999-2004, 2006-2015 Free Software Foundation, Inc.
+Contributed by the AriC and Caramel projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -35,19 +35,19 @@ int main (void)
   mpfr_t s, t, u;
 
   mpfr_init2 (t, 200);
-  mpfr_set_d (t, 1.0, GMP_RNDD);
+  mpfr_set_d (t, 1.0, MPFR_RNDD);
   mpfr_init2 (s, 200);
-  mpfr_set_d (s, 1.0, GMP_RNDD);
+  mpfr_set_d (s, 1.0, MPFR_RNDD);
   mpfr_init2 (u, 200);
   for (i = 1; i <= 100; i++)
     {
-      mpfr_mul_ui (t, t, i, GMP_RNDU);
-      mpfr_set_d (u, 1.0, GMP_RNDD);
-      mpfr_div (u, u, t, GMP_RNDD);
-      mpfr_add (s, s, u, GMP_RNDD);
+      mpfr_mul_ui (t, t, i, MPFR_RNDU);
+      mpfr_set_d (u, 1.0, MPFR_RNDD);
+      mpfr_div (u, u, t, MPFR_RNDD);
+      mpfr_add (s, s, u, MPFR_RNDD);
     }
   printf ("Sum is ");
-  mpfr_out_str (stdout, 10, 0, s, GMP_RNDD);
+  mpfr_out_str (stdout, 10, 0, s, MPFR_RNDD);
   putchar ('\n');
   mpfr_clear (s);
   mpfr_clear (t);
